/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.log.mongodb;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.InputStream;
import java.net.UnknownHostException;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.Mongo;

import de.flapdoodle.embed.mongo.MongodExecutable;
import de.flapdoodle.embed.mongo.MongodProcess;
import de.flapdoodle.embed.mongo.MongodStarter;
import de.flapdoodle.embed.mongo.config.AbstractMongoConfig.Net;
import de.flapdoodle.embed.mongo.config.MongodConfig;
import de.flapdoodle.embed.mongo.distribution.Version;
import de.flapdoodle.embed.process.runtime.Network;

/**
 * 
 * @author Christophe HAMERLING - Linagora
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MongoDbHandlerTest {

    private final static String MONGODB_LOCALHOST = "localhost";

    /**
     * Listen port of the mongoDb servers used by unit tests
     */
    private final static int MONGODB_PORT = 12345;

    /**
     * Name of the MongoDb database where logs are stored
     */
    private final static String PETALS_DB = "petals";

    /**
     * Naame of the MongoDb collection where logs are stored
     */
    private final static String PETALS_DB_COLLECTION = "petalslogs";

    /**
     * MongoDb executable
     */
    private MongodExecutable mongodExe;

    /**
     * MongoDb server
     */
    private MongodProcess mongod;

    @BeforeClass
    public static void beforeOne() throws UnknownHostException {
        org.ow2.petals.commons.log.Level.initialize();
    }

    @Before
    public void beforeEach() throws Exception {
        LogManager.getLogManager().reset();

        final MongodConfig mongodConfig = new MongodConfig(Version.V2_0_6, new Net(MONGODB_PORT,
                Network.localhostIsIPv6()), new MongodConfig.Storage(), new MongodConfig.Timeout(
                10000));
        
        final MongodStarter runtime = MongodStarter.getDefaultInstance();
        try {
            this.mongodExe = runtime.prepare(mongodConfig);
            this.mongod = this.mongodExe.start();
        } catch (final Exception e) {
            throw new RuntimeException(e);
        }
    }

    @After
    public void afterEach() throws Exception {
        if (this.mongod != null) {
            this.mongod.stop();
            this.mongodExe.stop();
        }
    }

    /**
     * Check that a trace with MONIT level is flushed in the Mongo DB.
     */
    @Test
    public void testLog_Level_MONIT() throws Exception {
        final LogManager manager = LogManager.getLogManager();
        final InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("loggers.properties");
        try {
            assertNotNull("Log file configuration not found", is);
            manager.readConfiguration(is);
        } finally {
            is.close();
        }

        final Logger logger = Logger.getLogger(MongoDbHandlerTest.class.getName());

        final String expectedMsg = "This is a test message #" + UUID.randomUUID();
        final Level expectedLogLevel = org.ow2.petals.commons.log.Level.MONIT;
        logger.log(expectedLogLevel, expectedMsg);

        final Mongo mongoc = new Mongo(MongoDbHandlerTest.MONGODB_LOCALHOST,
                MongoDbHandlerTest.MONGODB_PORT);
        try {
            final DBCollection mongoCol = mongoc.getDB(PETALS_DB).getCollection(
                    PETALS_DB_COLLECTION);
            final DBCursor cursor = mongoCol.find();

            assertTrue("Empty collection", cursor.hasNext());
            final DBObject object = cursor.next();
            assertFalse("More than one item in collection", cursor.hasNext());

            assertTrue("The log level is missing in the record",
                    object.containsField(MongoDbHandler.LOGGER_LEVEL));
            final Object fieldLogLevel = object.get(MongoDbHandler.LOGGER_LEVEL);
            assertTrue("Unexpected type of log level in the record",
                    fieldLogLevel instanceof String);
            assertEquals("Unexpected log level in the record", (String) fieldLogLevel,
                    expectedLogLevel.getName());

            final Object fieldLogMsg = object.get(MongoDbHandler.LOGGER_MESSAGE);
            assertTrue("Unexpected type of log message in the record",
                    fieldLogMsg instanceof String);
            assertEquals("Unexpected log message in the record", (String) fieldLogMsg, expectedMsg);

            final Object fieldTimeStamp = object.get(MongoDbHandler.LOGGER_TIME_MS);
            assertTrue("Unexpected type of log timestamp in the record",
                    fieldTimeStamp instanceof Long);
            assertNotNull("Empty log timestamp", fieldTimeStamp);

        } finally {
            mongoc.close();
        }
    }

    /**
     * Check that a trace with INFO level is not flushed in the Mongo DB.
     */
    @Test
    public void testLog_Level_INFO() throws Exception {
        final LogManager manager = LogManager.getLogManager();
        final InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("loggers.properties");
        try {
            assertNotNull("Log file configuration not found", is);
            manager.readConfiguration(is);
        } finally {
            is.close();
        }

        final Logger logger = Logger.getLogger(MongoDbHandlerTest.class.getName());

        final String expectedMsg = "This is a test message #" + UUID.randomUUID();
        final Level expectedLogLevel = Level.INFO;
        logger.log(expectedLogLevel, expectedMsg);

        final Mongo mongoc = new Mongo(MongoDbHandlerTest.MONGODB_LOCALHOST,
                MongoDbHandlerTest.MONGODB_PORT);
        try {
            final DBCollection mongoCol = mongoc.getDB(PETALS_DB).getCollection(
                    PETALS_DB_COLLECTION);
            final DBCursor cursor = mongoCol.find();

            assertFalse("Not empty collection", cursor.hasNext());

        } finally {
            mongoc.close();
        }
    }

    /**
     * Check that a trace with CONFIG level is not flushed in the Mongo DB.
     */
    @Test
    public void testLog_Level_CONFIG() throws Exception {
        final LogManager manager = LogManager.getLogManager();
        final InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("loggers.properties");
        try {
            assertNotNull("Log file configuration not found", is);
            manager.readConfiguration(is);
        } finally {
            is.close();
        }

        final Logger logger = Logger.getLogger(MongoDbHandlerTest.class.getName());

        final String expectedMsg = "This is a test message #" + UUID.randomUUID();
        final Level expectedLogLevel = Level.CONFIG;
        logger.log(expectedLogLevel, expectedMsg);

        final Mongo mongoc = new Mongo(MongoDbHandlerTest.MONGODB_LOCALHOST,
                MongoDbHandlerTest.MONGODB_PORT);
        try {
            final DBCollection mongoCol = mongoc.getDB(PETALS_DB).getCollection(
                    PETALS_DB_COLLECTION);
            final DBCursor cursor = mongoCol.find();

            assertFalse("Not empty collection", cursor.hasNext());

        } finally {
            mongoc.close();
        }
    }

    /**
     * Check that a trace with INFO level is not flushed in the Mongo DB.
     */
    @Test
    public void testLog_Level_FINE() throws Exception {
        final LogManager manager = LogManager.getLogManager();
        final InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("loggers.properties");
        try {
            assertNotNull("Log file configuration not found", is);
            manager.readConfiguration(is);
        } finally {
            is.close();
        }

        final Logger logger = Logger.getLogger(MongoDbHandlerTest.class.getName());

        final String expectedMsg = "This is a test message #" + UUID.randomUUID();
        final Level expectedLogLevel = Level.FINE;
        logger.log(expectedLogLevel, expectedMsg);

        final Mongo mongoc = new Mongo(MongoDbHandlerTest.MONGODB_LOCALHOST,
                MongoDbHandlerTest.MONGODB_PORT);
        try {
            final DBCollection mongoCol = mongoc.getDB(PETALS_DB).getCollection(
                    PETALS_DB_COLLECTION);
            final DBCursor cursor = mongoCol.find();

            assertFalse("Not empty collection", cursor.hasNext());

        } finally {
            mongoc.close();
        }
    }
}
